/* FLO_WAR.C : tkeress Floyd-Warshall mdszerrel */

#include <stdio.h>
#include <conio.h>

#define MaxPontDb 5                      /* A pontok maximlis szma */
#define MaxElDb MaxPontDb*(MaxPontDb-1)  /* Az lek maximlis szma */
#define MaxElHossz 99                    /* A maximlis lhossz */
#define NincsEl -1                       /* A nemltez l jelzsre */
#define MaxUtHossz MaxElDb*MaxElHossz    /* A maximlis thossz */
#define Vegtelen MaxUtHossz+1            /* A 'vgtelen' kezelshez */
#define KellKiiras 1                     /* Legyenek-e ellenrz kirsok */

typedef struct {                         /* A pontrekord */
  char Azon;                             /* Pontazonost */
  int x,y;                               /* Koordintk */
} Pont;

typedef int Matrix[MaxPontDb][MaxPontDb];  /* A mtrixos trolshoz */

typedef struct {                         /* A grf */
  int PontDb;                            /* Pontok szma */
  Pont Pontok[MaxPontDb];                /* Pontok */
  Matrix ElHossz;                        /* lhossz mtrix */
} Graf;

void MtxKiir(char *Szoveg, Matrix a, int n)
{ int i,j;
  printf("%s\n",Szoveg);
  for (i=0; i<n; i++) {
    for (j=0; j<n; j++)
      if (a[i][j]==Vegtelen)
	printf("%6c",'-');
      else
	printf("%6d",a[i][j]);
    printf("\n");
  }
}

void Floyd_Warshall(Graf *G, Matrix T, Matrix C)
{ int x,y,w;
  /* Kezdllapot */
  for (x=0; x<(*G).PontDb; x++)
    for (y=0; y<(*G).PontDb; y++) {
      if (x==y)
	T[x][y]=0;
      else
	if ((*G).ElHossz[x][y]==NincsEl)
	  T[x][y]=Vegtelen;
	else
	  T[x][y]=(*G).ElHossz[x][y];
      C[x][y]=y;
    }
  if (KellKiiras) {
    printf("A kezdllapot\n");
    MtxKiir("T:",T,(*G).PontDb);
    MtxKiir("C:",C,(*G).PontDb);
    getch();
  }
  /* Javt lpsek */
  for (w=0; w<(*G).PontDb; w++) {
    for (x=0; x<(*G).PontDb; x++)
      for (y=0; y<(*G).PontDb; y++)
	if (T[x][w]+T[w][y]<T[x][y]) {
	  T[x][y]=T[x][w]+T[w][y];
	  C[x][y]=C[x][w];
	}
    if (KellKiiras) {
      printf("%d. javt lps utn:\n",w+1);
      MtxKiir("T:",T,(*G).PontDb);
      MtxKiir("C:",C,(*G).PontDb);
      getch();
    }
  }
}

void GrafKiir(Graf *G)
{ int i,j,k;
  printf("Pontok szma:%d\n",(*G).PontDb);
  printf("Index Az-     X     Y\n");
  for (i=0; i<(*G).PontDb; i++)
    printf("%5d %c    %5d %5d\n",i+1,(*G).Pontok[i].Azon,
	   (*G).Pontok[i].x,(*G).Pontok[i].y);
  getch();
  printf("A grf lei:\n");
  for (i=0, k=1; i<(*G).PontDb; i++)
    for (j=0; j<(*G).PontDb; j++)
      if (i!=j && (*G).ElHossz[i][j]!=NincsEl) {
	printf("%c - %c : %d\n",(*G).Pontok[i].Azon,
	       (*G).Pontok[j].Azon,(*G).ElHossz[i][j]);
	if (k++%20==0) getch();
      }
  getch();
}

void UtKiir(Graf *G, Matrix T, Matrix C, int Kp, int Vp)
{ int x;
  printf("%c - %c viszonylat\n",(*G).Pontok[Kp].Azon,(*G).Pontok[Vp].Azon);
  if (T[Kp][Vp]==Vegtelen)
    printf("Nem lehet eljutni!\n");
  else {
    printf("A tvolsg:%d\n",T[Kp][Vp]);
    printf("Az t:");
    x=Kp;
    printf("%c ",(*G).Pontok[x].Azon);
    do {
      x=C[x][Vp];
      printf("%c ",(*G).Pontok[x].Azon);
    } while (x!=Vp);
    printf("\n");
    getch();
  }
}

void main()
{ /* A pldagrf mtrixos trolsban */
  Graf G = {5,{{'1',0,20},{'2',15,20},{'3',8,12},{'4',0,0},{'5',15,0}},
	   {{0,15,10,NincsEl,NincsEl},{NincsEl,0,NincsEl,NincsEl,20},
	   {10,10,0,13,NincsEl},{20,NincsEl,NincsEl,0,15},
	   {NincsEl,20,13,NincsEl,0}}};
  Matrix T,C;
  clrscr();
  if (KellKiiras)
    GrafKiir(&G);

  Floyd_Warshall(&G,T,C);

  /* A 2. pontbl a 4. pontba */
  UtKiir(&G,T,C,1,3);  /* 0-tl indexelnk */
}
